﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace NQueens
{
    internal class NQueensSolver
    {
        public int Solutions { get; private set; }

        public NQueensSolver(int boardSize)
        {
            int[,] board = new int[boardSize, boardSize];
            Iterate(board);
        }

        private void Iterate(int[,] board, int column = 0)
        {
            // Assume a square board.
            int size = board.GetLength(0);
            if (column == size)
            {
                //We have a solution
                Solutions++;
                Console.WriteLine($"Solution {Solutions}: ");
                PrintBoard(board);
                return;
            }

            for (int row = 0; row < size; row++)
            {
                if (RowIsSafe(board, row, column))
                {
                    if (DiagonalsAreSafe(board, row, column))
                    {
                        board[row, column] = 1;
                        Iterate(board, column + 1);
                        board[row, column] = 0;
                    }
                }
            }
        }

        private bool RowIsSafe(int[,] board, int row, int column)
        {
            // We only need to check up until the current column.
            // This is so because we place from left to right.
            // So every time we place a queen, we place it to
            // the right most available spot, there are no queens
            // further to the right, so we don't have to check it.
            for (int currentColumn = 0; currentColumn < column; currentColumn++)
            {
                if (board[row, currentColumn] != 0)
                {
                    return false;
                }
            }
            return true;
        }

        private bool DiagonalsAreSafe(int[,] board, int row, int column)
        {
            // We only need to check towards the left diagonals.
            // This is so because we place from left to right.
            // So every time we place a queen, we place it to
            // the right most available spot, there are no queens
            // further to the right, so we don't have to check it.
            int rowOffset = 1;
            for (int currentColumn = column-1; currentColumn >= 0; currentColumn--)
            {
                int rowUp = row - rowOffset;
                int rowDown = row + rowOffset;

                if (rowUp < 0 && rowDown >= board.GetLength(0))
                {
                    break;
                }

                if (rowUp >= 0)
                {
                    if (board[rowUp, currentColumn] != 0)
                    {
                        return false;
                    }
                }

                if (rowDown < board.GetLength(0))
                {
                    if (board[rowDown, currentColumn] != 0)
                    {
                        return false;
                    }
                }
                rowOffset++;
            }

            return true;
        }

        private void PrintBoard(int[,] board)
        {
            for (int row = 0; row < board.GetLength(0); row++)
            {
                for (int column = 0; column < board.GetLength(1); column++)
                {
                    if (board[row, column] == 1)
                    {
                        Console.Write("><");
                    }
                    else if ((row + column) % 2 == 0)
                    {
                        Console.Write("░░");
                    }
                    else
                    {
                        Console.Write("▓▓");
                    }
                }
                Console.WriteLine();
            }
        }
    }
}
